#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.


from psym.client import SymphonyClient
from psym.common.data_class import DocumentCategory
from .location_type import get_location_type_by_name
from psym.common.data_enum import Entity
from psym.exceptions import EntityNotFoundError


def get_document_category_by_names(
    client: SymphonyClient, location_type_name: str, doc_category_name: str
) -> DocumentCategory:
    """This function gets existing Document Category by its name.

    :param location_type_name:  Location type name
    :type location_type_name: str
    :param doc_category_name:  Document Category name
    :type doc_category_name: str

    :raises:
        * FailedOperationException: Internal symphony error
        * :class:`~psym.exceptions.EntityNotFoundError`: Document Category does not exist

    :return: DocumentCategory
    :rtype: :class:`~psym.common.data_class.DocumentCategory`

    **Example**

    .. code-block:: python

        client.get_doc_category_by_names(location_type_name="COUNTRY", doc_category_name="DATAFILLS")
    """

    location_type = get_location_type_by_name(client, name=location_type_name)

    result = None
    for item in location_type.document_categories:
        if item.name == doc_category_name:
            result = item

    if result is None:
        raise EntityNotFoundError(
            entity=Entity.DocumentCategory, entity_name=doc_category_name
        )

    return result



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class ProjectPriority(Enum):
    HIGH = "HIGH"
    LOW = "LOW"
    MEDIUM = "MEDIUM"
    NONE = "NONE"
    URGENT = "URGENT"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "ProjectPriority":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from .equipment_port_definition import EquipmentPortDefinitionFragment, QUERY as EquipmentPortDefinitionFragmentQuery

from .link import LinkFragment, QUERY as LinkFragmentQuery

from .property import PropertyFragment, QUERY as PropertyFragmentQuery

# fmt: off
QUERY: List[str] = EquipmentPortDefinitionFragmentQuery + LinkFragmentQuery + PropertyFragmentQuery + ["""
fragment EquipmentPortFragment on EquipmentPort {
  id
  properties {
    ...PropertyFragment
  }
  definition {
    ...EquipmentPortDefinitionFragment
  }
  link {
    ...LinkFragment
  }
}

"""]

@dataclass(frozen=True)
class EquipmentPortFragment(DataClassJsonMixin):
    @dataclass(frozen=True)
    class Property(PropertyFragment):
        pass

    @dataclass(frozen=True)
    class EquipmentPortDefinition(EquipmentPortDefinitionFragment):
        pass

    @dataclass(frozen=True)
    class Link(LinkFragment):
        pass

    id: str
    properties: List[Property]
    definition: EquipmentPortDefinition
    link: Optional[Link]



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.filter_operator import FilterOperator
from ..enum.rule_type_filter_type import RuleTypeFilterType


@dataclass(frozen=True)
class RuleTypeFilterInput(DataClassJsonMixin):
    filterType: RuleTypeFilterType = _field(metadata=enum_field_metadata(RuleTypeFilterType))
    idSet: List[str]
    operator: FilterOperator = _field(metadata=enum_field_metadata(FilterOperator))
    stringSet: List[str]
    maxDepth: Optional[int] = None
    stringValue: Optional[str] = None



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.survey_question_type import SurveyQuestionType


@dataclass(frozen=True)
class SurveyTemplateQuestionInput(DataClassJsonMixin):
    index: int
    questionDescription: str
    questionTitle: str
    questionType: SurveyQuestionType = _field(metadata=enum_field_metadata(SurveyQuestionType))
    id: Optional[str] = None



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from gql.transport.exceptions import TransportServerError
from psym.api.recommendations import remove_recommendations

from psym.common.constant import __version__
from psym.graphql.mutation.remove_recommendations import removeRecommendations

from .api.equipment_type import _populate_equipment_types
from .api.location_type import _populate_location_types
from .api.port_type import _populate_equipment_port_types
from .api.project_type import _populate_project_types
from .api.service_type import _populate_service_types
from .api.work_order_type import _populate_work_order_types
from .client import SymphonyClient


"""Psym is a python package that allows for querying and modifying the Symphony data using graphql queries.

This module contains the client that allows to connect to inventory. The client
allows different kind of operations: querying and creating of locations, equipments,
positions and links.

Example of how to connect:
```
from psym import PsymClient
# since symphony is multi tenant system you will need to insert which
# partner you connect as
client = PsymClient(email, password, "tenant_name")
location = client.add_location(
    location_hirerchy=[
        ("Country", "England"),
        ("City", "Milton Keynes"),
        ("Site", "Bletchley Park")
    ],
    properties_dict={
        "Date Property": date.today(),
        "Lat/Lng Property": (-1.23,9.232),
        "E-mail Property": "user@fb.com",
        "Number Property": 11,
        "String Property": "aa",
        "Float Property": 1.23,
    },
    lat=-11.32,
    long=98.32,
    external_id=None
)
equipment = client.add_equipment(
    name="Router X123",
    equipment_type="Router",
    location=location,
    properties_dict={
        "Date Property": date.today(),
        "Lat/Lng Property": (-1.23,9.232),
        "E-mail Property": "user@fb.com",
        "Number Property": 11,
        "String Property": "aa",
        "Float Property": 1.23,
    }
)
```
"""


class UserDeactivatedException(Exception):
    pass
class PsymClient(SymphonyClient):
    from.api.counter_family import (
        add_counter_family,
        edit_counter_family,
        get_counter_families,
        remove_counter_family,
    )
    from.api.counter_formula import (add_counter_formula,)
    from.api.services import (get_services,)

    from.api.counter import (
        add_counter,
        edit_counter,
        get_counters,
        remove_counter,
    )
    from.api.domain import (
        add_domain,
        edit_domain,
        get_domains,
        remove_domain,
    )
    from.api.event_severity import (
        add_event_severity,
        edit_event_severity,
        get_event_severities,
        remove_event_severity,
    )
    from.api.formula import (
        add_formula,
        edit_formula,
        get_formulas,
        remove_formula,
    )
    from.api.kpi_category import (
        add_kpi_category,
        get_kpi_categories,
        delete_kpi_category,
        edit_kpi_category,
    )
    from.api.kpi import (
        add_kpi,
        edit_kpi,
        get_kpis,
        remove_kpi,
    )
    from.api.kqi_category import (
        add_kqi_category,
        edit_kqi_category,
        delete_kqi_category,
        get_kqi_categories,
    )
    from.api.kqi_perspective import (
        add_kqi_perspective,
        edit_kqi_perspective,
        delete_kqi_perspective,
        get_kqi_perspectives,

    )
    from.api.kqi_source import (
        add_kqi_source,
        edit_kqi_source,
        delete_kqi_source,
        get_kqi_sources,

    )
    from.api.kqi_temporal_frecuency import (
            add_kqi_temporal_frecuency,
            edit_kqi_temporal_frecuency,
            delete_kqi_temporal_frecuency,
            get_kqi_temporal_frecuencies,

        )
    from.api.network_type import (
            add_network_type,
            edit_network_type,
            get_network_types,
            remove_network_type,
        )
    from.api.recommendation_category import (
            add_recommendations_category,
            edit_recommendations_category,
            get_recommendations_categories,
            remove_recommendations_category,
        )
    from.api.recommendation_sources import (
            add_recommendations_sources,
            edit_recommendations_sources,
            get_recommendations_sources,
            remove_recommendations_sources,
        )
    from.api.recommendations import (
            add_recommendations, 
            edit_recommendations,
            get_recommendations,
            remove_recommendations
            )
    from.api.rule_limit import (
            add_rule_limit,
            edit_rule_limit,
            remove_rule_limit,
            )
    from.api.rule_type import (
            add_rule_type,
            edit_rule_type,
            get_rule_types,
            remove_rule_type,

        )
    from.api.rule import (
            add_rule,
            )
    from.api.tech import (
            add_tech,
            edit_tech,
            get_tech,
            remove_tech,
        )
    from.api.threshold import (
            add_threshold,
            edit_threshold,
            get_thresholds,
            remove_threshold,
        )
    from.api.vendor import (
            add_vendor,
            edit_vendor,
            get_vendors,
            remove_vendor,
        )
    from.api.comparator import (
        add_comparator,
        edit_comparator,
        get_comparators,
        remove_comparator,)
    from.api.alarm_status import (
        add_alarm_status,
        edit_alarm_status,
        get_alarm_statuses,
        remove_alarm_status,
        )
    from.api.organization import (
        add_organization, 
        edit_organization, 
        get_organizations, 
        remove_organization,
        get_organization_by_name,
        )
    from .api.file import add_location_image, delete_document, add_file, add_files
    from .api.document_category import get_document_category_by_names
    from .api.location_type import (
        add_location_type,
        add_property_types_to_location_type,
        delete_location_type,
        get_location_types,
        get_location_type_by_id,
        get_location_type_by_name,
        edit_location_type,
        delete_locations_by_location_type,
        delete_location_type_with_locations,
    )
    from .api.location import (
        get_location,
        get_location_by_external_id,
        get_location_children,
        get_location_documents,
        delete_location,
        add_location,
        edit_location,
        move_location,
        get_locations,
        get_location_by_name,
    )
    from .api.equipment_type import (
        copy_equipment_type,
        delete_equipment_type_with_equipments,
        _add_equipment_type,
        add_equipment_type,
        add_property_types_to_equipment_type,
        get_or_create_equipment_type,
        _edit_equipment_type,
        edit_equipment_type,
        get_equipment_type_property_type,
        get_equipment_type_property_type_by_external_id,
        edit_equipment_type_property_type,
        delete_equipment_type,
        get_equipment_type_by_id,
        get_equipment_types,
        get_equipment_types_by_name,
    )
    from .api.equipment import (
        add_equipment,
        add_equipment_to_position,
        get_equipment,
        get_equipment_in_position,
        get_equipments,
        delete_equipment,
        search_for_equipments,
        delete_all_equipments,
        copy_equipment_in_position,
        copy_equipment,
        get_equipment_type_of_equipment,
        get_or_create_equipment,
        get_or_create_equipment_in_position,
        edit_equipment,
        get_equipment_properties,
        get_equipments_by_type,
        get_equipments_by_location,
        get_equipment_by_external_id,
    )
    from .api.link import (
        add_link,
        get_link_in_port_of_equipment,
        get_links,
        get_all_links_and_port_names_of_equipment,
    )
    from .api.service import (
        add_service,
        add_service_endpoint,
        add_service_link,
        get_service,
    )
    from .api.service_type import (
        add_service_type,
        add_property_types_to_service_type,
        get_service_type,
        get_service_types,
        edit_service_type,
        delete_service_type,
        delete_service_type_with_services,
    )
    from .api.location_template import (
        apply_location_template_to_location,
        copy_equipment_with_all_attachments,
    )
    from .api.customer import add_customer, delete_customer, get_all_customers
    from .api.port_type import (
        add_equipment_port_type,
        get_equipment_port_type,
        edit_equipment_port_type,
        delete_equipment_port_type,
        get_equipment_port_types,
    )
    from .api.port import (
        get_port,
        edit_port_properties,
        edit_link_properties,
        get_ports,
    )
    from .api.user import (
        add_user,
        get_user,
        edit_user_for_password_and_role,
        edit_user,
        deactivate_user,
        activate_user,
        get_users,
        get_active_users,
        get_user_by_email,
    )
    from .api.property_type import (
        get_property_type_id,
        get_property_types,
        get_property_type,
        get_property_type_by_external_id,
    )

    from .api.features import get_enabled_features, set_feature
    from .api.work_order_type import (
        get_work_order_type_by_name,
        get_work_order_type_by_id,
        get_work_order_types,
        add_work_order_type,
        add_property_types_to_work_order_type,
        delete_work_order_type,
    )
    from .api.work_order import (
        add_work_order,
        get_work_orders,
        get_work_order_by_id,
        get_work_order_by_name,
        edit_work_order,
        delete_work_order,
    )
    from .api.project_type import (
        add_project_type,
        add_property_types_to_project_type,
        delete_project_type,
        edit_project_type,
        get_project_type_by_id,
        get_project_types,
    )
    from .api.project import (
        add_project,
        delete_project,
        edit_project,
        get_project_by_id,
        get_projects,
    )

    def __init__(
        self,
        email: str,
        password: str,
        tenant: str = "fb-test",
        is_local_host: bool = False,
        is_dev_mode: bool = False,
    ) -> None:
        """This is the class to use for working with inventory. It contains all
        the functions to query and and edit the inventory.

        The __init__ method populates the different entity types
        for faster run of operations.

        Args:
            email (str): The email of the user to connect with.
            password (str): The password of the user to connect with.
            tenant (str, optional): The tenant to connect to -
                        should be the beginning of "{}.purpleheadband.cloud"
                        The default is "fb-test" for QA environment
            is_local_host (bool, optional): Used for developers to connect to
                        local inventory. This changes the address and also
                        disable verification of ssl certificate
            is_dev_mode (bool, optional): Used for developers to connect to
                        local inventory from a container. This changes the
                        address and also disable verification of ssl
                        certificate

        """
        super().__init__(
            email,
            password,
            tenant,
            f"Psym/{__version__}",
            is_local_host,
            is_dev_mode,
        )
        try:
            self.populate_types()
        except TransportServerError as e:
            err_msg = str(e.args[0])
            if "Forbidden" in err_msg:
                raise UserDeactivatedException()
            raise

    def populate_types(self) -> None:
        _populate_location_types(self)
        _populate_equipment_types(self)
        _populate_service_types(self)
        _populate_equipment_port_types(self)
        _populate_work_order_types(self)
        _populate_project_types(self)



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from .customer import CustomerFragment, QUERY as CustomerFragmentQuery

from .property import PropertyFragment, QUERY as PropertyFragmentQuery

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.service_status import ServiceStatus

# fmt: off
QUERY: List[str] = CustomerFragmentQuery + PropertyFragmentQuery + ["""
fragment ServiceFragment on Service {
  id
  name
  externalId
  status
  serviceType {
    id
    name
  }
  customer {
    ...CustomerFragment
  }
  properties {
    ...PropertyFragment
  }
}

"""]

@dataclass(frozen=True)
class ServiceFragment(DataClassJsonMixin):
    @dataclass(frozen=True)
    class ServiceType(DataClassJsonMixin):
        id: str
        name: str

    @dataclass(frozen=True)
    class Customer(CustomerFragment):
        pass

    @dataclass(frozen=True)
    class Property(PropertyFragment):
        pass

    id: str
    name: str
    externalId: Optional[str]
    status: ServiceStatus = _field(metadata=enum_field_metadata(ServiceStatus))
    serviceType: ServiceType
    customer: Optional[Customer]
    properties: List[Property]



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from typing import Optional

from psym.common.data_enum import Entity
from requests.models import Response


def assert_ok(resp: Response) -> None:
    if not resp.ok:
        error_message = resp.json().get("error", None)
        if error_message is not None:
            raise AssertionError(error_message)
        raise AssertionError()


class CustomException(Exception):
    pass


def get_location_id_msg(
    location_name: Optional[str] = None,
    location_type: Optional[str] = None,
    external_id: Optional[str] = None,
) -> str:
    msg = "location"
    if location_name:
        msg = msg + f" {location_name}"
    if location_type:
        msg = msg + f" of type {location_type}"
    if external_id:
        msg = msg + f" with external id {external_id}"
    return msg


class EntityNotFoundError(Exception):
    def __init__(
        self,
        entity: Entity,
        entity_id: Optional[str] = None,
        entity_name: Optional[str] = None,
        entity_email: Optional[str] = None,
        msg: Optional[str] = None,
    ) -> None:
        self.entity: Entity = entity
        self.entity_id: str = entity_id if entity_id is not None else ""
        self.entity_name: str = entity_name if entity_name is not None else ""
        self.entity_email: str = entity_email if entity_email is not None else ""
        self.msg: str = msg if msg is not None else ""

    def __str__(self) -> str:
        message = f"{self.entity} entity"
        if self.msg:
            message = "\n".join([self.msg, f"{self.entity} entity"])
        if self.entity_name:
            message = " ".join([message, f"<enitity_name: {self.entity_name}>"])
        if self.entity_id:
            message = " ".join([message, f"<entity_id: {self.entity_id}>"])
        return " ".join([message, "does not found in inventory"])


class LocationNotFoundException(CustomException):
    def __init__(
        self,
        location_name: Optional[str] = None,
        location_type: Optional[str] = None,
        external_id: Optional[str] = None,
    ) -> None:
        self.location_name: Optional[str] = location_name
        self.location_type: Optional[str] = location_type
        self.external_id: Optional[str] = external_id
        msg = get_location_id_msg(location_name, location_type, external_id)
        msg = msg + " does not exist in inventory"
        super(LocationNotFoundException, self).__init__(msg)


class LocationIsNotUniqueException(CustomException):
    def __init__(
        self,
        location_name: Optional[str] = None,
        location_type: Optional[str] = None,
        external_id: Optional[str] = None,
    ) -> None:
        self.location_name: Optional[str] = location_name
        self.location_type: Optional[str] = location_type
        self.external_id: Optional[str] = external_id
        msg = get_location_id_msg(location_name, location_type, external_id)
        msg = msg + " has more than one result in inventory"
        super(LocationIsNotUniqueException, self).__init__(msg)


class EquipmentNotFoundException(CustomException):
    def __init__(
        self,
        equipment_name: Optional[str] = None,
        parent_equipment_name: Optional[str] = None,
        parent_position_name: Optional[str] = None,
    ) -> None:
        self.equipment_name: Optional[str] = equipment_name
        self.parent_equipment_name: Optional[str] = parent_equipment_name
        self.parent_position_name: Optional[str] = parent_position_name
        if equipment_name:
            super(EquipmentNotFoundException, self).__init__(
                f"equipment {equipment_name} does not exist in inventory"
            )
        else:
            super(EquipmentNotFoundException, self).__init__(
                f"Position {parent_equipment_name} in equipment "
                f"{parent_position_name} is not occupied"
            )


class EquipmentIsNotUniqueException(CustomException):
    def __init__(
        self,
        equipment_name: Optional[str] = None,
        parent_equipment_name: Optional[str] = None,
        parent_position_name: Optional[str] = None,
    ) -> None:
        self.equipment_name: Optional[str] = equipment_name
        self.parent_equipment_name: Optional[str] = parent_equipment_name
        self.parent_position_name: Optional[str] = parent_position_name
        if equipment_name:
            super(EquipmentIsNotUniqueException, self).__init__(
                f"equipment {equipment_name} has more than one result in inventory"
            )
        else:
            super(EquipmentIsNotUniqueException, self).__init__(
                f"More than one installed equipment in equipment "
                f"{parent_equipment_name} at position {parent_position_name}"
            )


class EquipmentPositionNotFoundException(CustomException):
    def __init__(self, parent_equipment_name: str, parent_position_name: str) -> None:
        self.parent_equipment_name: str = parent_equipment_name
        self.parent_position_name: str = parent_position_name
        super(EquipmentPositionNotFoundException, self).__init__(
            f"Equipment {parent_equipment_name} has no position "
            f"{parent_position_name}"
        )


class EquipmentPositionIsNotUniqueException(CustomException):
    def __init__(self, parent_equipment_name: str, parent_position_name: str) -> None:
        self.parent_equipment_name: str = parent_equipment_name
        self.parent_position_name: str = parent_position_name
        super(EquipmentPositionIsNotUniqueException, self).__init__(
            f"Equipment {parent_equipment_name} has more than one position "
            f"{parent_position_name}"
        )


class LinkNotFoundException(CustomException):
    def __init__(self, equipment_name: str, port_name: str) -> None:
        self.equipment_name: str = equipment_name
        self.port_name: str = port_name
        super(LinkNotFoundException, self).__init__(
            f"No link in port {port_name} in equipment {equipment_name}"
        )


class PortAlreadyOccupiedException(CustomException):
    def __init__(self, equipment_name: str, port_name: str) -> None:
        self.equipment_name: str = equipment_name
        self.port_name: str = port_name
        super(PortAlreadyOccupiedException, self).__init__(
            f"Port {port_name} in equipment {equipment_name} is already occupied"
        )


class EquipmentPortNotFoundException(CustomException):
    def __init__(self, equipment_name: str, port_name: str) -> None:
        self.equipment_name: str = equipment_name
        self.port_name: str = port_name
        super(EquipmentPortNotFoundException, self).__init__(
            f"Equipment {equipment_name} has no port {port_name}"
        )


class EquipmentPortIsNotUniqueException(CustomException):
    def __init__(self, equipment_name: str, port_name: str) -> None:
        self.equipment_name: str = equipment_name
        self.port_name: str = port_name
        super(EquipmentPortIsNotUniqueException, self).__init__(
            f"Equipment {equipment_name} has more than one port {port_name}"
        )


class LocationCannotBeDeletedWithDependency(CustomException):
    def __init__(self, location_name: str, dependency: str) -> None:
        self.location_name: str = location_name
        self.dependency: str = dependency
        super(LocationCannotBeDeletedWithDependency, self).__init__(
            f"Location {location_name} cannot be deleted with {dependency}"
        )



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class FlowInstanceStatus(Enum):
    CANCELED = "CANCELED"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"
    IN_PROGRESS = "IN_PROGRESS"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "FlowInstanceStatus":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class ServiceFilterType(Enum):
    EQUIPMENT_IN_SERVICE = "EQUIPMENT_IN_SERVICE"
    LOCATION_INST = "LOCATION_INST"
    LOCATION_INST_EXTERNAL_ID = "LOCATION_INST_EXTERNAL_ID"
    PROPERTY = "PROPERTY"
    SERVICE_DISCOVERY_METHOD = "SERVICE_DISCOVERY_METHOD"
    SERVICE_INST_CUSTOMER_NAME = "SERVICE_INST_CUSTOMER_NAME"
    SERVICE_INST_EXTERNAL_ID = "SERVICE_INST_EXTERNAL_ID"
    SERVICE_INST_NAME = "SERVICE_INST_NAME"
    SERVICE_STATUS = "SERVICE_STATUS"
    SERVICE_TYPE = "SERVICE_TYPE"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "ServiceFilterType":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.


from psym.client import SymphonyClient
from psym.common.data_class import domain, tech, tech

from ..graphql.input.edit_tech_input import EditTechInput
from ..graphql.input.add_tech_input import AddTechInput
from ..graphql.mutation.add_tech import addTech
from ..graphql.mutation.edit_tech import editTech
from ..graphql.mutation.remove_tech import removeTech
from ..graphql.query.techs import techs
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional




def add_tech(
    client: SymphonyClient, name: str, domain: str
) -> tech:
    """This function adds Tech.

    :param name: Name
    :type name: str
    :param domain: domain Object
    :type domain: psym.common.data_class.domain
    :return: Tech object
    :rtype: :class:`~psym.common.data_class.Tech`

    **Example 1**

    .. code-block:: python

        domain = client.add_domain(
            name="domain",
        )
        
        new_Tech = client.add_Tech(
            name="new_Tech", 
            domain=domain.id
            )
        print(new_Tech)
    """
    tech_input = AddTechInput(name=name, domainFk=domain)
    result = addTech.execute(client, input=tech_input)
    return tech(name=result.name, id=result.id, domainFK=result.domainFk)


def edit_tech(
    client: SymphonyClient,
    tech: tech,
    new_name: Optional[str] = None,
    domain: domain = None,
) -> None:
    """This function edits Tech.

    :param Tech: Tech entity
    :type name: str
    :param new_name: Tech name
    :type name: str

    :return: none object
    :rtype: :class:`~psym.common.data_class.Tech`

    **Example 1**

    .. code-block:: python


        Tech_edited = client.edit_Tech(
            Tech=Tech ,
            new_name="new_Tech",
            domain=domain.id)
    """
    params: Dict[str, Any] = {}
    if new_name is not None:
        params.update({"_name_": new_name})
    if new_name is not None:
        editTech.execute(client, input=EditTechInput(id=tech.id, name=new_name, domainFk=domain))

def get_tech(client: SymphonyClient) -> Iterator[tech]:
    """ this funtion Get Techs


    :return: Tech object
    :rtype: Iterator[ :class:`~psym.common.data_class.tech` ]

    **Example**

    .. code-block:: python

        Tech = client.get_techs()
        for Tech in Techs:
            print(Tech.name)
    """
    tech_ = techs.execute(client, first=PAGINATION_STEP)
    edges = tech_.edges if tech_ else []
    while tech_ is not None and tech_.pageInfo.hasNextPage:
        tech_ = techs.execute(
            client, after=tech_.pageInfo.endCursor, first=PAGINATION_STEP
        )
        if tech_ is not None:
            edges.extend(tech_.edges)
    for edge in edges:
        node = edge.node
        if node is not None:
            yield tech(
                id=node.id,
                name=node.name,
                domainFK=node.id

            )


def remove_tech(client: SymphonyClient, id: str) -> None:
    """This function delete Tech.

    :param name: Tech name
    :type name: :class:`~psym.common.data_class.Tech`
    :rtype: None

    **Example**

    .. code-block:: python

        client.remove_tech(id=123456789)
    """
    removeTech.execute(client, id=id)





#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from ..input.parent_property_type_value_input import ParentPropertyTypeValueInput


@dataclass(frozen=True)
class AddPropertyTypeValueInput(DataClassJsonMixin):
    name: str
    id: Optional[str] = None
    isDeleted: Optional[bool] = None
    parentPropertyType: Optional[List[ParentPropertyTypeValueInput]] = None
    parentPropertyTypeValue: Optional[List[str]] = None
    propertyType: Optional[str] = None



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.survey_question_type import SurveyQuestionType

from ..input.file_input import FileInput
from ..input.survey_cell_scan_data import SurveyCellScanData
from ..input.survey_wi_fi_scan_data import SurveyWiFiScanData


@dataclass(frozen=True)
class SurveyQuestionResponse(DataClassJsonMixin):
    cellData: List[SurveyCellScanData]
    formIndex: int
    imagesData: List[FileInput]
    questionIndex: int
    questionText: str
    wifiData: List[SurveyWiFiScanData]
    altitude: Optional[Number] = None
    boolData: Optional[bool] = None
    dateData: Optional[int] = None
    emailData: Optional[str] = None
    floatData: Optional[Number] = None
    formDescription: Optional[str] = None
    formName: Optional[str] = None
    intData: Optional[int] = None
    latitude: Optional[Number] = None
    locationAccuracy: Optional[Number] = None
    longitude: Optional[Number] = None
    phoneData: Optional[str] = None
    photoData: Optional[FileInput] = None
    questionFormat: Optional[SurveyQuestionType] = None
    textData: Optional[str] = None



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from ..input.edit_formula_input import EditFormulaInput


# fmt: off
QUERY: List[str] = ["""
mutation editFormula($input: EditFormulaInput!) {
  editFormula(input: $input) {
    id
    textFormula
    techFk{id}
    kpiFk{id}
    networkTypeFk{id}
    status
  }
}

"""
]


class editFormula:
    @dataclass(frozen=True)
    class editFormulaData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class Formula(DataClassJsonMixin):
            @dataclass(frozen=True)
            class Tech(DataClassJsonMixin):
                id: str

            @dataclass(frozen=True)
            class Kpi(DataClassJsonMixin):
                id: str

            @dataclass(frozen=True)
            class NetworkType(DataClassJsonMixin):
                id: str

            id: str
            textFormula: Optional[str]
            techFk: Tech
            kpiFk: Kpi
            networkTypeFk: NetworkType
            status: bool

        editFormula: Formula

    # fmt: off
    @classmethod
    def execute(cls, client: Client, input: EditFormulaInput) -> editFormulaData.Formula:
        variables: Dict[str, Any] = {"input": input}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.editFormulaData.from_dict(response_text)
        return res.editFormula

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, input: EditFormulaInput) -> editFormulaData.Formula:
        variables: Dict[str, Any] = {"input": input}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.editFormulaData.from_dict(response_text)
        return res.editFormula



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class RecommendationsFilterType(Enum):
    ALARMTYPE = "ALARMTYPE"
    COMMAND = "COMMAND"
    EXTERNALID = "EXTERNALID"
    LONGDESCRIPTION = "LONGDESCRIPTION"
    PRIORITY = "PRIORITY"
    RECOMMENDATIONSCATEGORY = "RECOMMENDATIONSCATEGORY"
    RECOMMENDATIONSSOURCE = "RECOMMENDATIONSSOURCE"
    RESOURCE = "RESOURCE"
    RUNBOOK = "RUNBOOK"
    SHORTDESCRIPTION = "SHORTDESCRIPTION"
    STATUS = "STATUS"
    USED = "USED"
    USERAPPROVE = "USERAPPROVE"
    USERCREATE = "USERCREATE"
    VENDORRECOMMENDATIONS = "VENDORRECOMMENDATIONS"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "RecommendationsFilterType":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config


@dataclass(frozen=True)
class AddCostInput(DataClassJsonMixin):
    externalId: str
    item: str
    position: int
    price: Number
    quantity: int
    total: Number
    type_price: str
    type_unit: str
    unit: Number
    uplitem: str
    id: Optional[str] = None
    isDeleted: Optional[bool] = None
    workorder: Optional[str] = None



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import math
from datetime import datetime
from itertools import combinations
from typing import Any, Dict, List, Optional, Sequence, Tuple, Union

import pandas as pd
from dacite import Config, from_dict
from psym.api.file import add_image, delete_image
from psym.common.constant import SIMPLE_QUESTION_TYPE_TO_REQUIRED_PROPERTY_NAME
from psym.common.data_class import SiteSurvey
from psym.client import SymphonyClient
from xlsxwriter.format import Format
from xlsxwriter.utility import xl_col_to_name
from xlsxwriter.workbook import Workbook
from xlsxwriter.worksheet import Worksheet

from ..graphql.enum.image_entity import ImageEntity
from ..graphql.enum.survey_question_type import SurveyQuestionType
from ..graphql.fragment.survey import SurveyFragment
from ..graphql.fragment.survey_question import SurveyQuestionFragment
from ..graphql.input.survey_create_data import SurveyCreateData
from ..graphql.input.survey_question_response import SurveyQuestionResponse
from ..graphql.mutation.create_survey import CreateSurveyMutation
from ..graphql.mutation.remove_site_survey import RemoveSiteSurveyMutation
from ..graphql.query.location_surveys import LocationSurveysQuery
from .site_survey_schema import retrieve_tamplates_and_set_them


def add_site_survey_image(
    client: SymphonyClient, local_file_path: str, id: str
) -> None:
    """This function adds image to existing site survey.

    Args:
        local_file_path (str): local system path to the file
        id (str): site survey ID

    Raises:
        FailedOperationException: on operation failure

    Example:
        ```
        client.add_site_survey_image(
            local_file_path="./document.pdf",
            id="123456"
        )
        ```
    """
    add_image(client, local_file_path, ImageEntity.SITE_SURVEY, id)


def delete_site_survey_image(client: SymphonyClient, survey: SiteSurvey) -> None:
    """This function deletes image from existing site survey.

    Args:
        survey ( `psym.common.data_class.SiteSurvey` ): site survey object

    Raises:
        FailedOperationException: on operation failure

    Example:
        ```
        client.delete_site_survey_image(survey=survey)
        ```
    """
    source_file_key = survey.source_file_key
    source_file_id = survey.source_file_id
    if source_file_key is not None:
        client.delete_file(source_file_key, False)
    if source_file_id is not None:
        delete_image(client, ImageEntity.SITE_SURVEY, survey.survey_id, source_file_id)


def _get_dependencies(question: Dict[str, Any]) -> Tuple[List[str], List[str]]:
    if "dependsOn" not in question:
        return [], []

    question_names = []  #  type: ignore
    dependency_names = []  #  type: ignore

    q = question
    while "dependsOn" in q:
        question_names.insert(0, q["dependsOn"]["questionName"])
        dependency_names.insert(0, q["dependsOn"]["name"])
        q = q["dependsOn"]

    return question_names, dependency_names


def _extract_questions(
    content: Dict[str, Any]
) -> Tuple[List[Tuple[str, ...]], List[Dict[str, Any]], int]:
    form_to_questions = [
        (form["formTitle"], list(form["questions"])) for form in content["forms"]
    ]
    full_question_paths = []
    questions = []
    max_deps = 0
    for form_to_question in form_to_questions:
        form_name = form_to_question[0]
        form_questions = form_to_question[1]
        for question in form_questions:
            question_names, question_deps = _get_dependencies(question)
            max_deps = max(max_deps, len(question_deps))
            full_question_paths.append(
                (
                    form_name,
                    *zip(question_deps, question_names),
                    question["questionName"],
                )
            )
            questions.append(question)

    return full_question_paths, questions, max_deps


def _extract_form_descriptions(content: Dict[str, Any]) -> Dict[str, str]:
    form_name_to_description = {}
    for form in content["forms"]:
        if "formDescription" in form:
            form_name_to_description[form["formTitle"]] = form["formDescription"]

    return form_name_to_description


def rpad(_list: List[str], char: str, size: int) -> List[str]:
    return _list + [char] * (size - len(_list))


def adjust_column_and_row_sizes(
    worksheet: Worksheet, rows: List[Tuple[str, ...]], num_columns: int
) -> None:
    """
    Rows height:
    - Title row is thicker than data row
    - blank separator row is thinner than data row
    Columns width:
    - Each column width is wider enough all the text in cells it contains

    :param worksheet: worksheet we operate on
    :param rows: list of tuples where each tuple represents a row in worksheet
    :param num_columns: number of columns in worksheet
    :return:
    """
    num_rows = len(rows)

    # adjust row sizes
    column_max_sizes = [0] * num_columns
    worksheet.set_row(0, 30)
    for i in range(num_rows):
        if rows[i][0] != " ":
            worksheet.set_row(i + 1, 20)
        else:
            worksheet.set_row(i + 1, 10)
        for j in range(num_columns):
            column_max_sizes[j] = max(column_max_sizes[j], len(rows[i][j]) + 5)

    # adjust column sizes
    for j in range(num_columns):
        worksheet.set_column(j, j, column_max_sizes[j])
    worksheet.set_column(num_columns, num_columns, 20)


def write_column_titles(
    worksheet: Worksheet,
    num_columns: int,
    title_format: Format,
    translations: Dict[str, str],
) -> None:
    worksheet.write("A1", translations.get("Form", "Form"), title_format)
    for i in range(1, num_columns - 1):
        worksheet.write_blank(f"{xl_col_to_name(i)}1", "", title_format)
    worksheet.write(
        f"{xl_col_to_name(num_columns - 1)}1",
        translations.get("Question", "Question"),
        title_format,
    )
    worksheet.write(
        f"{xl_col_to_name(num_columns)}1",
        translations.get("Answer", "Answer"),
        title_format,
    )


def write_conditional_blank_color_if_dependency_fails(
    worksheet: Worksheet,
    row_index: int,
    column_index: int,
    question: Dict[str, Any],
    full_question_path: Tuple[str, ...],
    question_to_cell: Dict[Tuple[str, ...], str],
    bool_options: Dict[str, str],
    black_format: Format,
) -> None:
    index = -1
    criterias = []
    q = question
    while "dependsOn" in q:
        compare, value = q["dependsOn"]["compare"], q["dependsOn"]["value"]
        new_question_path = full_question_path[:index]
        dependent_cell_name = question_to_cell[
            (
                new_question_path[0],
                *list(map(lambda x: x[0], list(new_question_path[1:-1]))),
                new_question_path[-1][1],
            )
        ]
        comparison_value = (
            f'"{bool_options["Yes"]}"'
            if value is True
            else '"{boolOptions["No"]}"'
            if value is False
            else f"{value}"
            if isinstance(value, int) or isinstance(value, float)
            else f'"{value}"'
        )
        opposite_compare = (
            "<>"
            if compare == "="
            else "="
            if compare == "<>"
            else ">"
            if compare == "<="
            else "<="
            if compare == ">"
            else ">="
            if compare == "<"
            else "<"
        )
        criterias.append(
            'AND(${0}{1}{2},${0}<>"")'.format(
                dependent_cell_name, opposite_compare, comparison_value
            )
        )
        index = index - 1
        q = q["dependsOn"]
    if len(criterias) > 1:
        final_criteria = "=OR("
        for i, criteria in enumerate(criterias):
            if i == 0:
                final_criteria = final_criteria + criteria
            else:
                final_criteria = final_criteria + "," + criteria
        final_criteria = final_criteria + ")"
    else:
        final_criteria = "=" + criterias[0]
    worksheet.conditional_format(
        row_index,
        column_index,
        row_index,
        column_index,
        {"type": "formula", "criteria": final_criteria, "format": black_format},
    )


def write_form_description_as_cell_comment(
    worksheet: Worksheet,
    form_names: List[str],
    question_index_to_row_index: Dict[int, int],
    form_name_to_description: Dict[str, str],
) -> None:
    # add form description as comment to column and make every other form
    # appear in different color
    last_form_name = ""
    i = 1
    for form_name in form_names:
        if form_name != last_form_name:
            last_form_name = form_name
            if form_name in form_name_to_description:
                worksheet.write_comment(
                    question_index_to_row_index[i + 1],
                    0,
                    form_name_to_description[form_name],
                )
        i = i + 1


def add_workbook_formats(workbook: Workbook) -> Tuple[Format, Format, Format, Format]:
    title_format = workbook.add_format(
        {"bg_color": "#DCDCDC", "bold": True, "align": "center", "valign": "vcenter"}
    )
    date_format = workbook.add_format(
        {
            "num_format": "dd/mm/yyyy",
            "locked": 0,
            "align": "center",
            "valign": "vcenter",
        }
    )
    black_format = workbook.add_format({"bg_color": "#000000"})
    cell_format = workbook.add_format(
        {"locked": 0, "align": "center", "valign": "vcenter"}
    )

    return title_format, date_format, black_format, cell_format


def add_cell_validation(
    worksheet: Worksheet,
    row_index: int,
    column_index: int,
    full_question_path: Tuple[str, ...],
    question: Dict[str, Any],
    question_to_cell: Dict[Tuple[str, ...], str],
    bool_options: Dict[str, str],
    date_format: Format,
    black_format: Format,
) -> None:
    type = question["questionType"]
    has_options = "options" in question
    if type == "BOOL":
        worksheet.data_validation(
            row_index,
            column_index,
            row_index,
            column_index,
            {"validate": "list", "source": list(bool_options.values())},
        )
    elif has_options:
        options = question["options"]
        if "multiOptions" in question and question["multiOptions"]:
            s = set(options)
            multi_options = sum(  #  type: ignore
                map(lambda r: sorted(list(combinations(s, r))), range(1, len(s) + 1)),
                [],
            )  #  type: ignore
            options = [";".join(multi_option) for multi_option in multi_options]
        worksheet.data_validation(
            row_index,
            column_index,
            row_index,
            column_index,
            {"validate": "list", "source": options},
        )
    elif type == "INTEGER":
        worksheet.data_validation(
            row_index,
            column_index,
            row_index,
            column_index,
            {"validate": "integer", "criteria": ">", "value": -(2 ** 32)},
        )
    elif type == "FLOAT":
        worksheet.data_validation(
            row_index,
            column_index,
            row_index,
            column_index,
            {"validate": "decimal", "criteria": ">", "value": -(2 ** 32)},
        )
    elif type == "DATE":
        worksheet.write_datetime(
            row_index,
            column_index,
            datetime.strptime("01/01/1970", "%d/%m/%Y"),
            date_format,
        )
    if "questionDescription" in question:
        worksheet.write_comment(
            row_index, column_index, question["questionDescription"]
        )
    if "dependsOn" in question:
        write_conditional_blank_color_if_dependency_fails(
            worksheet,
            row_index,
            column_index,
            question,
            full_question_path,
            question_to_cell,
            bool_options,
            black_format,
        )


def export_to_excel(json_file_path: str, excel_file_path: str) -> None:
    content = retrieve_tamplates_and_set_them(json_file_path)

    # pyre-fixme[45]: Cannot instantiate abstract class `ExcelWriter`.
    writer = pd.ExcelWriter(excel_file_path, engine="xlsxwriter")
    workbook = writer.book
    assert workbook is not None

    title_format, date_format, black_format, cell_format = add_workbook_formats(
        workbook
    )

    translations = content.get("translations", {})
    bool_options = {
        "Yes": translations.get("Yes", "Yes"),
        "No": translations.get("No", "No"),
    }
    for category in content["categories"]:
        full_question_paths, questions, max_deps = _extract_questions(category)
        form_name_to_description = _extract_form_descriptions(category)

        rows = [
            (
                full_question_path[0],
                *rpad(
                    list(map(lambda x: x[0], list(full_question_path[1:-1]))),
                    " ",
                    max_deps,
                ),
                full_question_path[-1],
            )
            for full_question_path in full_question_paths
        ]
        df = pd.DataFrame({row: [] for row in rows})

        for i in range(len(rows) - 1, -1, -1):
            if i != 0 and rows[i - 1][0] != rows[i][0]:
                df.insert(
                    i,
                    column=tuple([" "] * (max_deps + 2)),
                    value="",
                    allow_duplicates=True,
                )
        df = df.transpose()
        df.to_excel(writer, sheet_name=category["categoryName"])

        rows = [record for record in df.to_records()]
        row_indexes = [i + 2 for i, row in enumerate(rows) if row[0] != " "]
        question_index_to_row_index = dict(
            zip(range(2, len(row_indexes) + 2), row_indexes)
        )

        num_columns = max_deps + 2
        worksheet = writer.sheets[category["categoryName"]]
        write_column_titles(worksheet, num_columns, title_format, translations)
        adjust_column_and_row_sizes(worksheet, rows, num_columns)

        question_to_cell = {}

        # add validations and formatting to columns
        for i, (full_question_path, question) in enumerate(
            zip(full_question_paths, questions)
        ):
            cell_name = "{}{}".format(
                xl_col_to_name(num_columns), question_index_to_row_index[i + 2]
            )
            question_to_cell[
                (
                    full_question_path[0],
                    *list(map(lambda x: x[0], list(full_question_path[1:-1]))),
                    full_question_path[-1],
                )
            ] = cell_name

            worksheet.write_blank(cell_name, "", cell_format)
            # pyre-fixme[19]: Expected 9 positional arguments.
            add_cell_validation(  #  type: ignore
                worksheet,
                question_index_to_row_index[i + 2],
                num_columns,
                question_to_cell,  #  type: ignore
                full_question_path,  #  type: ignore
                question,  #  type: ignore
                question_to_cell,  #  type: ignore
                bool_options,
                date_format,
                black_format,
            )

        form_names = [
            full_question_path[0] for full_question_path in full_question_paths
        ]
        write_form_description_as_cell_comment(
            worksheet, form_names, question_index_to_row_index, form_name_to_description
        )

        worksheet.protect()

    writer.save()


def _nan_to_empty_string(value: Union[float, int, str]) -> str:
    return "" if isinstance(value, float) and math.isnan(value) else str(value)


def dependency_check(
    compare: str,
    value: Union[float, int, str, bool],
    result: str,
    bool_options: Dict[str, str],
) -> bool:
    if isinstance(value, bool):
        assert compare in ["=", "<>"]
        if compare == "=":
            return (result == bool_options["Yes"] and value is True) or (
                result == bool_options["No"] and value is False
            )
        else:
            return (result == bool_options["Yes"] and value is False) or (
                result == bool_options["No"] and value is True
            )
    elif isinstance(value, int) or isinstance(value, float):
        float_result = float(result)
        if compare == ">=":
            return float_result >= value
        elif compare == ">":
            return float_result > value
        elif compare == "<":
            return float_result < value
        elif compare == "<=":
            return float_result <= value
        elif compare == "<>":
            return float_result != value
        else:
            return float_result == value
    else:
        assert compare in ["=", "<>"]
        if compare == "=":
            return result == value
        else:
            return result != value


def break_and_validate_coordinates(
    value: str, invalid_type_err_msg: str
) -> Tuple[float, float]:
    value_list = value.strip('"').split(",")
    assert len(value_list) == 2, invalid_type_err_msg
    try:
        new_value = (float(value_list[0]), float(value_list[1]))
    except ValueError:
        raise AssertionError(invalid_type_err_msg)
    assert (
        new_value[0] >= -90 and new_value[0] <= 90
    ), "Latitude is not between -90 and 90 ({})".format(new_value[0])
    assert (
        new_value[1] >= -90 and new_value[1] <= 90
    ), "Longtitude is not between -90 and 90 ({})".format(new_value[1])
    return new_value


def get_response_value(
    full_representation_path: Tuple[str, ...],
    question_type: str,
    value: str,
    bool_options: Dict[str, str],
) -> Dict[str, Any]:
    invalid_type_err_msg = "Value of {} ({}) is not of type {}".format(
        full_representation_path, value, question_type
    )

    if question_type == "COORDS":
        lat, long = break_and_validate_coordinates(value, invalid_type_err_msg)
        return {"latitude": lat, "longitude": long}
    elif question_type == "BOOL":
        assert value in list(bool_options.values()), invalid_type_err_msg
        return {"boolData": True if value == bool_options["Yes"] else False}
    elif question_type == "EMAIL":
        return {"emailData": str(value)}
    elif question_type == "TEXT":
        return {"textData": str(value)}
    elif question_type == "FLOAT":
        try:
            return {"floatData": float(value)}
        except ValueError:
            raise AssertionError(invalid_type_err_msg)
    elif question_type == "INTEGER":
        try:
            return {"intData": int(value)}
        except ValueError:
            raise AssertionError(invalid_type_err_msg)
    elif question_type == "DATE":
        return {
            "dateData": int(datetime.strptime(value, "%Y-%m-%d %H:%M:%S").timestamp())
        }
    elif question_type == "PHONE":
        return {"phoneData": str(value)}
    raise AssertionError(f"question type {question_type} not found")


def get_survey_response(
    category_name: str,
    form_name: str,
    form_index: int,
    full_representation_path: Tuple[str, ...],
    question_index: int,
    question_type: str,
    value: str,
    bool_options: Dict[str, str],
    form_description: Optional[str] = None,
) -> Dict[str, Any]:
    response = {
        "formName": category_name + " - " + form_name,
        "formIndex": form_index,
        "questionText": " - ".join(full_representation_path[1:]),
        "questionFormat": SurveyQuestionType(question_type),
        "questionIndex": question_index,
        "wifiData": [],
        "cellData": [],
        "imagesData": [],
    }

    if form_description:
        response.update({"formDescription": form_description})

    response_value = get_response_value(
        full_representation_path, question_type, value, bool_options
    )
    response.update(response_value)

    return response


def _get_survey_reponses(
    excel_file_path: str, json_file_path: str
) -> List[Dict[str, Any]]:
    content = retrieve_tamplates_and_set_them(json_file_path)

    responses = []

    form_index = -1

    translations = content.get("translations", {})
    bool_options = {
        "Yes": translations.get("Yes", "Yes"),
        "No": translations.get("No", "No"),
    }
    for category in content["categories"]:
        results = pd.read_excel(
            excel_file_path, sheet_name=category["categoryName"]
        ).values.tolist()
        results = [result for result in results if result[0] != " "]
        results = [_nan_to_empty_string(result[-1]) for result in results]
        full_question_paths, questions, max_deps = _extract_questions(category)

        results = {
            (
                full_question_path[0],
                *list(map(lambda x: x[0], list(full_question_path[1:-1]))),
                full_question_path[-1],
            ): results[i]
            for i, full_question_path in enumerate(full_question_paths)
        }
        form_name_to_description = _extract_form_descriptions(category)

        full_representation_path_to_question = {
            (
                full_question_path[0],
                *list(map(lambda x: x[0], list(full_question_path[1:-1]))),
                full_question_path[-1],
            ): question
            for (full_question_path, question) in zip(full_question_paths, questions)
        }

        last_form_name = ""
        question_index = -1
        for full_question_path in full_question_paths:
            full_representation_path = (
                full_question_path[0],
                *list(map(lambda x: x[0], list(full_question_path[1:-1]))),
                full_question_path[-1],
            )
            form_name = full_question_path[0]
            if form_name != last_form_name:
                last_form_name = form_name
                form_index = form_index + 1
                question_index = 0
            else:
                question_index = question_index + 1

            if len(full_question_path) > 2:
                dependencies_checks = []  #  type: ignore
                depends_on = full_representation_path_to_question[
                    full_representation_path
                ]
                q_path = full_question_path
                while len(q_path) > 2:
                    depends_on = depends_on["dependsOn"]
                    compare, value = depends_on["compare"], depends_on["value"]
                    result = results[
                        (
                            q_path[:-1][0],
                            *list(map(lambda x: x[0], list(q_path[:-1][1:-1]))),
                            q_path[:-1][-1][1],
                        )
                    ]
                    dependencies_checks.insert(0, (compare, value, result))
                    q_path = q_path[:-1]

                dependency_failed = False
                for compare, value, result in dependencies_checks:
                    if not dependency_check(compare, value, result, bool_options):
                        dependency_failed = True
                        break
                if dependency_failed:
                    continue

            question = full_representation_path_to_question[full_representation_path]

            value = results[full_representation_path]
            if "options" in question:
                options = question["options"]
                if "multi_options" in question:
                    s = set(options)
                    multi_options = sum(  #  type: ignore
                        map(
                            lambda r: sorted(list(combinations(s, r))),
                            range(1, len(s) + 1),
                        ),
                        [],
                    )
                    options = [";".join(multi_option) for multi_option in multi_options]
                assert value in options, "Value of {} ({}) not in options: {}".format(
                    full_representation_path, value, options
                )

            question_type = question["questionType"]

            response = get_survey_response(
                category["categoryName"],
                form_name,
                form_index,
                full_representation_path,
                question_index,
                question_type,
                value,
                bool_options,
                form_name_to_description.get(form_name, None),
            )

            responses.append(response)

    return responses


def upload_site_survey(
    client: SymphonyClient,
    location_id: str,
    name: str,
    completion_date: datetime,
    excel_file_path: str,
    json_file_path: str,
) -> None:
    """Upload the site survey to the given completion with the data in the
    given excel file. We use the schema file to validate the input in the
    excel is as needed for upload.

    Args:
        location ( `psym.common.data_class.Location` ): could be retrieved from getLocation or addLocation api
        name (str): name of the site survey
        completion_date (datetime.datetime object): the time the site survey was completed
        excel_file_path (str): the path for the excel with the site survey information
                            The format of this excel should be created by calling site_survey.exportToExcel
                            with the paremeter jsonFilePath (the next parameter of this function)
        json_file_path(str): the path for the json file of the schema of the site survey
                           the json file should comply to the schema found in survey_schema.json
                           Example of the format:
        ```
        {
            "forms": [
            {
                "formTitle": "Site Management - General Information",
                "questions": [
                {
                    "questionName": "Exact address",
                    "questionType": "TEXT"
                },
                {
                    "questionName": "Reference for address",
                    "questionType": "TEXT"
                },
                {
                    "questionName": "Ubigeo",
                    "questionType": "TEXT"
                }
                ]
            }
            ]
        }
        ```

    Raises:
        AssertionException: if input values in the excel are incorrect
        FailedOperationException: Internal symphony error
    """

    site_survey_id = CreateSurveyMutation.execute(
        client,
        data=SurveyCreateData(
            name=name,
            completionTimestamp=int(datetime.timestamp(completion_date)),
            locationID=location_id,
            surveyResponses=[
                from_dict(
                    data_class=SurveyQuestionResponse,
                    data=response,
                    config=Config(strict=True),
                )
                for response in _get_survey_reponses(excel_file_path, json_file_path)
            ],
        ),
    )
    add_site_survey_image(client, excel_file_path, site_survey_id)


def _survey_responses_to_forms(
    responses: Sequence[SurveyQuestionFragment],
) -> Dict[str, Dict[str, Any]]:
    forms = {}  #  type: ignore
    value_does_not_match_type_error_msg = (
        "{} question type doesn't have the correct response type {}"
    )
    for response in responses:
        form_name = response.formName
        if form_name not in forms:
            forms[form_name] = {}
        form = forms[form_name]
        question_text = response.questionText
        question_type = response.questionFormat
        if question_type == "COORDS":
            assert (
                response.latitude is not None and response.longitude is not None
            ), value_does_not_match_type_error_msg.format(
                "COORDS", "[latitude, longitude]"
            )
            form[question_text] = (response.latitude, response.longitude)
        elif question_type == "DATE":
            date_data = response.dateData
            assert date_data is not None, value_does_not_match_type_error_msg.format(
                "DATE", "dateData"
            )
            form[question_text] = datetime.fromtimestamp(date_data)
        else:
            for (
                simple_question_type,
                required_property,
            ) in SIMPLE_QUESTION_TYPE_TO_REQUIRED_PROPERTY_NAME.items():
                if question_type == simple_question_type:
                    assert (
                        response.__dict__[required_property] is not None
                    ), value_does_not_match_type_error_msg.format(
                        simple_question_type, required_property
                    )
                    form[question_text] = response.__dict__[required_property]
                    if question_type == "DATE":
                        form[question_text] = datetime.fromtimestamp(
                            form[question_text]
                        )
                    break

    return forms


def build_site_survey_from_survey_response(survey: SurveyFragment) -> SiteSurvey:
    survey_id = survey.id
    name = survey.name
    completion_time = datetime.fromtimestamp(survey.completionTimestamp)
    source_file = survey.sourceFile
    source_file_id = source_file.id if source_file else None
    source_file_name = source_file.fileName if source_file else None
    source_file_key = source_file.storeKey if source_file else None
    forms = _survey_responses_to_forms(survey.surveyResponses)

    return SiteSurvey(
        survey_id=survey_id,
        name=name,
        completion_time=completion_time,
        source_file_id=source_file_id,
        source_file_name=source_file_name,
        source_file_key=source_file_key,
        forms=forms,
    )


def get_site_surveys(client: SymphonyClient, location_id: str) -> List[SiteSurvey]:
    """Retrieve all site survey completed in the location.

    Args:
        location ( `psym.common.data_class.Location` ): could be retrieved from getLocation or addLocation api

    Returns:
        List[ `psym.common.data_class.SiteSurvey` ]

    Raises:
        `psym.exceptions.EntityNotFoundError`: location does not exist
    """

    location_with_surveys = LocationSurveysQuery.execute(client, id=location_id)
    if not location_with_surveys:
        raise Exception(f"Location not found. location id: {location_id}")
    surveys = location_with_surveys.surveys
    return [build_site_survey_from_survey_response(survey) for survey in surveys]


def delete_site_survey(client: SymphonyClient, site_survey: SiteSurvey) -> None:
    delete_site_survey_image(client, site_survey)
    RemoveSiteSurveyMutation.execute(client, id=site_survey.survey_id)



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class YesNoResponse(Enum):
    NO = "NO"
    YES = "YES"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "YesNoResponse":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.


from psym.client import SymphonyClient
from psym.common.data_class import Kpi, threshold

from ..graphql.input.edit_threshold_input import EditThresholdInput
from ..graphql.input.add_threshold_input import AddThresholdInput
from ..graphql.mutation.add_threshold import addThreshold
from ..graphql.mutation.edit_threshold import editThreshold
from ..graphql.mutation.remove_threshold import removeThreshold
from ..graphql.query.thresholds import thresholds
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional




def add_threshold(
    client: SymphonyClient, name: str, description: str, status: bool ,kpi: str
) -> threshold:
    """This function adds Threshold.

    :param name: Threshold name
    :type name: str
    :param description: description
    :type description: str
    :param status: status
    :type status: bool
    :param kpi: str
    :type kpi: psym.common.data_class.kpi

    :return: Threshold object
    :rtype: :class:`~psym.common.data_class.Threshold`

    **Example 1**

    .. code-block:: python

        domain = client.add_domain(
            name="domain",
        )
        kpi_category = client.add_kpi_category(
            name="kpi_category",
        )

        Kpi = client.add_Kpi(
            name="Kpi_2",
            description="new kpi",
            status=True,
            domain=domain.id,
            kpiCategory=kpi_category.id
        )

        new_Threshold = client.add_Threshold(
            name="Threshold",
            description="threshold",
            status=True,
            kpi=Kpi.id
        )
    """
    threshold_input = AddThresholdInput(name=name, description=description, status=status, kpi=kpi)
    result = addThreshold.execute(client, input=threshold_input)
    return threshold(name=result.name, description=result.description, status=result.status, id=result.id, kpi=result.kpi)


def edit_threshold(
    client: SymphonyClient,
    threshold: threshold,
    new_name: Optional[str] = None,
    description: str = None,
    status: bool = None,
) -> None:
    """This function edits Threshold.

    :param Threshold: Threshold entity
    :type name: str
    :param new_name: Threshold name
    :type name: str

    :return: none object
    :rtype: :class:`~psym.common.data_class.Threshold`

    **Example 1**

    .. code-block:: python

        Threshold_edited = client.edit_threshold(
            threshold=Threshold,
            new_name=new_name,
            description="threshold",
            status=True
            )
    """
    params: Dict[str, Any] = {}
    if new_name is not None:
        params.update({"_name_": new_name})
    if new_name is not None:
        editThreshold.execute(client, input=EditThresholdInput(id=threshold.id, 
        name=new_name, 
        description=description,
        status=status))

def get_thresholds(client: SymphonyClient) -> Iterator[threshold]:
    """ this funtion Get Thresholds


    :return: Threshold object
    :rtype: Iterator[ :class:`~psym.common.data_class.Threshold` ]

    **Example**

    .. code-block:: python

        Thresholds = client.get_Thresholds()
        for threshold in Thresholds:
            print(threshold.name)
    """
    threshold_ = thresholds.execute(client, first=PAGINATION_STEP)
    edges = threshold_.edges if threshold_ else []
    while threshold_ is not None and threshold_.pageInfo.hasNextPage:
        threshold_ = thresholds.execute(
            client, after=threshold_.pageInfo.endCursor, first=PAGINATION_STEP
        )
        if threshold_ is not None:
            edges.extend(threshold_.edges)

    for edge in edges:
        node = edge.node
        if node is not None:
            yield threshold(
                id=node.id,
                name=node.name,
                description=node.description,
                status=node.status,
                kpi=node.id

            )


def remove_threshold(client: SymphonyClient, id: str) -> None:
    """This function delete Threshold.

    :param name: Threshold name
    :type name: :class:`~psym.common.data_class.Threshold`
    :rtype: None

    **Example**

    .. code-block:: python

        client.remove_threshold(id=123456789)
    """
    removeThreshold.execute(client, id=id)





#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from ..input.action_block_input import ActionBlockInput
from ..input.connector_input import ConnectorInput
from ..input.decision_block_input import DecisionBlockInput
from ..input.end_block_input import EndBlockInput
from ..input.goto_block_input import GotoBlockInput
from ..input.start_block_input import StartBlockInput
from ..input.subflow_block_input import SubflowBlockInput
from ..input.trigger_block_input import TriggerBlockInput
from ..input.true_false_block_input import TrueFalseBlockInput
from ..input.variable_definition_input import VariableDefinitionInput


@dataclass(frozen=True)
class ImportFlowDraftInput(DataClassJsonMixin):
    actionBlocks: List[ActionBlockInput]
    connectors: List[ConnectorInput]
    decisionBlocks: List[DecisionBlockInput]
    endBlocks: List[EndBlockInput]
    endParamDefinitions: List[VariableDefinitionInput]
    gotoBlocks: List[GotoBlockInput]
    id: str
    name: str
    subflowBlocks: List[SubflowBlockInput]
    triggerBlocks: List[TriggerBlockInput]
    trueFalseBlocks: List[TrueFalseBlockInput]
    description: Optional[str] = None
    startBlock: Optional[StartBlockInput] = None



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class PropertyKind(Enum):
    bool = "bool"
    date = "date"
    datetime_local = "datetime_local"
    email = "email"
    enum = "enum"
    float = "float"
    gps_location = "gps_location"
    int = "int"
    node = "node"
    range = "range"
    string = "string"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "PropertyKind":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.cellular_network_type import CellularNetworkType


@dataclass(frozen=True)
class SurveyCellScanData(DataClassJsonMixin):
    networkType: CellularNetworkType = _field(metadata=enum_field_metadata(CellularNetworkType))
    signalStrength: int
    altitude: Optional[Number] = None
    arfcn: Optional[int] = None
    baseStationID: Optional[str] = None
    cellID: Optional[str] = None
    earfcn: Optional[int] = None
    heading: Optional[Number] = None
    latitude: Optional[Number] = None
    locationAreaCode: Optional[str] = None
    longitude: Optional[Number] = None
    mobileCountryCode: Optional[str] = None
    mobileNetworkCode: Optional[str] = None
    networkID: Optional[str] = None
    operator: Optional[str] = None
    physicalCellID: Optional[str] = None
    primaryScramblingCode: Optional[str] = None
    rssi: Optional[Number] = None
    systemID: Optional[str] = None
    timestamp: Optional[int] = None
    timingAdvance: Optional[int] = None
    trackingAreaCode: Optional[str] = None
    uarfcn: Optional[int] = None
